from django.core.management.base import BaseCommand
from django.conf import settings
import os

import django_po_translator.app_settings as app_settings


class Command(BaseCommand):
    help = 'Restore a po file to its original format'

    def handle(self, *args, **options):
        
        languages = []
        
        for language in settings.LANGUAGES:
            if os.path.exists(os.path.join(os.getcwd(), 'locale', language[0], 'LC_MESSAGES', app_settings.PO_FILES_NAME)):
                languages.append(language[0])
        
        for language in languages:
            po_path = os.path.join(os.getcwd(), 'locale', language, 'LC_MESSAGES', app_settings.PO_FILES_NAME)
            # save a backup
            po_backup_path = os.path.join(os.getcwd(), 'locale', language, 'LC_MESSAGES', 'django.po.bak')
            po_clean_path = os.path.join(os.getcwd(), 'locale', language, 'LC_MESSAGES', 'django.po.clean')

            os.system(f'cp {po_path} {po_backup_path}')
            os.system(f'msgattrib --no-obsolete --no-location -o {po_clean_path} {po_path}')
            os.system(f'cp {po_clean_path} {po_path}')


import urllib.parse
import urllib.request
from bs4 import BeautifulSoup
import django_po_translator.app_settings as app_settings

def translate_text(text, target_language):
    
    base_url = app_settings.GOOGLE_TRANSLATOR_BASE_URL
    url = base_url + 'm?hl=en&sl=auto&tl=' + urllib.parse.quote(target_language) + '&ie=UTF-8&prev=_m&q=' + urllib.parse.quote(text)
    
    response = urllib.request.urlopen(url)
    soup = BeautifulSoup(response, 'html.parser')
    
    translated_text = soup.find('div', {'class': app_settings.GOOGLE_RESULT_DIV_CONTAINER_CLASS}).text
    
    return translated_text


from django.conf import settings


  
GOOGLE_TRANSLATOR_BASE_URL = getattr(settings, 'GOOGLE_TRANSLATOR_BASE_URL',  'https://translate.google.com/') 
GOOGLE_RESULT_DIV_CONTAINER_CLASS = getattr(settings, 'GOGGLE_RESULT_DIV_CONTAINER_CLASS', 'result-container')
PO_FILES_NAME = getattr(settings, 'PO_FILES_NAME', 'django.po')



import os
from django.core.management import BaseCommand
from django.conf import settings  

from django_po_translator.processors import lines_processor, action

class Command(BaseCommand):
    
    help = 'Manage Django PO files.'
    
    def add_arguments(self, parser):
        parser.add_argument('--translate-existed', dest='translate-existed', action='store_true', 
                            help='Translate and update already existed translations, the current translation will be overridden')
        
        parser.add_argument('--resolve-fuzzy', dest='resolve-fuzzy', action='store_true', 
                            help='Remove all fuzziness by re-translating the msgid and remove #, fuzzy keyword')


    def handle(self, *args, **options):
        
        translate_existed = options.get('translate-existed', False)
        resolve_fuzzy = options.get('resolve-fuzzy', False)
        
        action(command=self, translate_existed=translate_existed, 
                            resolve_fuzzy=resolve_fuzzy)
                


from django import test
import os 

from django_po_translator.translate import translate_text
from django_po_translator.processors import lines_processor, action


class TestPoTranslator(test.TestCase):
                
     def test_translate_ep(self):

          translated_text = translate_text(text="Hello",target_language="ar")
          self.assertIsNotNone(translated_text)
          
          
     def test_action_processor(self):
          lines = None
          po_test_file_path = os.path.abspath(os.path.join(os.path.dirname(__file__), 'test_po.po'))
          with open(po_test_file_path, 'r', encoding='utf-8') as po_file:
               lines = po_file.readlines()
               
          self.assertIsNotNone(lines)
          
          
     def test_line_processor(self):
          
          lines = None
          po_test_file_path = os.path.abspath(os.path.join(os.path.dirname(__file__), 'test_po.po'))
          with open(po_test_file_path, 'r', encoding='utf-8') as po_file:
               lines = po_file.readlines()
               
          res, lines = lines_processor(lines, 'ar', update_already_translated=True, resolve_fuzzy=True)
          
          self.assertTrue(res)
          self.assertNotEqual(len(lines), 0)
          
          
     def test_line_processor_after_fixing_fuzziness(self):
          
          lines = None
          po_test_file_path = os.path.abspath(os.path.join(os.path.dirname(__file__), 'test_po.po'))
          with open(po_test_file_path, 'r', encoding='utf-8') as po_file:
               lines = po_file.readlines()
          
          self.assertEqual(lines.count("#, fuzzy\n"), 2)
          res, lines = lines_processor(lines, 'ar', update_already_translated=True, resolve_fuzzy=True)
          
          self.assertTrue(res)
          self.assertNotEqual(len(lines), 0)
          self.assertEqual(lines.count("#, fuzzy\n"), 1) # only the one in the title
          
          
     def test_line_processor_without_fixing_fuzziness(self):
          
          lines = None
          po_test_file_path = os.path.abspath(os.path.join(os.path.dirname(__file__), 'test_po.po'))
          with open(po_test_file_path, 'r', encoding='utf-8') as po_file:
               lines = po_file.readlines()
          
          self.assertEqual(lines.count("#, fuzzy\n"), 2)
          res, lines = lines_processor(lines, 'ar', update_already_translated=True, resolve_fuzzy=False)
          
          self.assertTrue(res)
          self.assertNotEqual(len(lines), 0)
          self.assertEqual(lines.count("#, fuzzy\n"), 2) # only the one in the title
          
          
     def test_line_processor_without_override_translations(self):
          
          lines = None
          po_test_file_path = os.path.abspath(os.path.join(os.path.dirname(__file__), 'test_po.po'))
          with open(po_test_file_path, 'r', encoding='utf-8') as po_file:
               lines = po_file.readlines()
          
          # This is a wrong translation .. added intentionally
          self.assertEqual(lines.count('msgstr "صحيح"\n'), 1)
            
          res, lines = lines_processor(lines, 'ar', update_already_translated=False, resolve_fuzzy=True)
          
          # wrong translation should be stick in there .. without any change
          self.assertEqual(lines.count('msgstr "صحيح"\n'), 1)
        
     
     def test_line_processor_with_override_translations(self):
          
          lines = None
          po_test_file_path = os.path.abspath(os.path.join(os.path.dirname(__file__), 'test_po.po'))
          with open(po_test_file_path, 'r', encoding='utf-8') as po_file:
               lines = po_file.readlines()
          
          # This is a wrong translation .. added intentionally
          self.assertEqual(lines.count('msgstr "صحيح"\n'), 1)
            
          res, lines = lines_processor(lines, 'ar', update_already_translated=True, resolve_fuzzy=True)
          
          # wrong translation should be overridden
          self.assertEqual(lines.count('msgstr "صحيح"\n'), 0)
     
     


import os
from django.conf import settings
from django.core.management import BaseCommand
from .translate import translate_text
from halo import Halo

import django_po_translator.app_settings as app_settings



def lines_processor(lines, lan, update_already_translated=False, resolve_fuzzy=False):
    """
    parse msgid and msgid and apply the translation to a new list
    
    return, processed list.
    """
    
    backup_lines = lines
    processed_lines = lines
    try:
        processed_lines = []
        for index, line in enumerate(lines):
            if is_msgstr(line):
                # update even if the msgstr already provided
                if update_already_translated:
                    processed_lines = translate_msgstr_line_as_list(processed_lines, lines, index, lan)                    
                else:
                    msgstr = get_str(line)
                    if msgstr == '':
                        processed_lines = translate_msgstr_line_as_list(processed_lines, lines, index, lan)
                    else:
                        # msgstr already provided ... keep it as it is
                        processed_lines.append(line)
            else:
                processed_lines.append(line)
        
        # We will handle the fuzziness after the whole translation is done ... to reduce the complexity
        # it will increase the time of executing ... but to avoid un-calculated actions   
        if resolve_fuzzy:
            processed_lines = clear_fuzziness(processed_lines, lan)
        
        return True, processed_lines
    except:
        # In case failed ... return the backup tp prevent any lose of data.
        return False, backup_lines


def translate_msgstr_line_as_list(processed_lines:list, lines:list, index, lan):
    """ Apply translation and return updated list """ 
    msgid = get_msgid(lines, msgstr_index=index)
    translated_text = fetch_translation(text=msgid, lan=lan)
    processed_lines.append('msgstr "' + translated_text + '"\n')
    
    return processed_lines


def clear_fuzziness(lines:list, lan):
    """ Check fuzzy records and translate and remove #, fuzzy """
    cleaned_list = []
    indexes_to_be_removed = []
    for index, line in enumerate(lines):
        if is_fuzzy(lines, index):
            if is_msgstr(line):
                msgid = get_msgid(lines, msgstr_index=index)
                translated_text = fetch_translation(text=msgid, lan=lan)
                cleaned_list.append('msgstr "' + translated_text + '"\n')
                # todo: remove the #, fuzzy       
                
                indexes_to_be_removed.append(index - 2)
            else:
                cleaned_list.append(line)
        else:
            cleaned_list.append(line)
            
    if indexes_to_be_removed:
        cleaned_list = [cleaned_list[i] for i in range(len(cleaned_list)) if i not in indexes_to_be_removed[1:]]
  
    return cleaned_list
    

def is_msgid(line) -> bool:
    """ 
    Return if the line is message id 
    
    return: bool
    """
    try:
        if line.split(" ")[0] == "msgid":
            return True
        return False
    except:
        return False


def is_msgstr(line):
    """ 
    Return if the line is message str
    
    return: bool
    """
    try:
        if line.split(" ")[0] == "msgstr":
            return True
        return False
    except:
        return False
    

def get_str(line):
    """ Return the string is exist and None if the msgstr is empty """
    if is_msgstr(line): # TODO remove if not necessary
        return line.split('"')[1] if line.split('"')[1] != '"' else None


def get_msgid(lines, msgstr_index):
    """ Return the msgid of a given msgstr index """
    try:
        return lines[msgstr_index - 1].split('"')[1] if lines[msgstr_index - 1].split('"')[1] != '"' else None
    except:
        raise ValueError("Not valid msgstr index.")
    
    
def fetch_translation(text, lan):
    """ Translate the giving word """
    if text != "":
        # Translate some text
        result = translate_text(text=text, target_language=lan)
        return result if result else ''
    
    return ''


def is_fuzzy(lines, msgstr_index):
    """ Determine whether this translation is fuzzy """
    try:
        return lines[msgstr_index - 2].split("\n")[0] == "#, fuzzy"
    except:
        return False


def action(command:BaseCommand, translate_existed, resolve_fuzzy):
    """ 
    Apply the actions into the PO files
    
    args 
    - command: the command that is calling this function
    - translated_existed: bool: process already translated words
    - resolve_fuzzy: bool: remove and fix all fuzzy words
    
    return bool, string
    """
    
    command.stdout.write(command.style.SUCCESS("\nInitial new process ... \n\n"))
    
    if translate_existed:
        command.stdout.write(command.style.WARNING("NOTE: Existing translations will overridden"))
    else:
        command.stdout.write(command.style.WARNING("NOTE: Existing translations will not overridden, if you want to override pass --translate-existed"))
        
    if resolve_fuzzy:
        command.stdout.write(command.style.WARNING("NOTE: Fuzzy translations will be resolved"))
    else:
        command.stdout.write(command.style.WARNING("NOTE: Fuzzy translations will be not resolved, if you want to resolve them pass --resolve-fuzzy"))
        
    command.stdout.write("\nRun make messages \n")
    
    result = os.system('python manage.py makemessages -a')
    
    if os.WEXITSTATUS(result) != 0:
        command.stdout.write(command.style.ERROR("Failed to make messages .. please fix the issue mentioned above then try again \n\n"))
        return
    else:
        command.stdout.write(command.style.SUCCESS("make messages ... Ok \n\n"))
    
    command.stdout.write("Reformatting PO files \n")
    os.system('python manage.py restore_po_formatting')
    command.stdout.write(command.style.SUCCESS("Reformatting ... Ok \n\n"))
    
    languages = None
    try:
        languages = [language[0] for language in settings.LANGUAGES]
    except:
        return False, "No languages detect in the project, make sure LANGUAGES is defined in project settings"
    
    po_files_paths = []
    for language in languages: 
        if os.path.exists(os.path.join(os.getcwd(), 'locale', language, 'LC_MESSAGES', app_settings.PO_FILES_NAME)):
            po_file = os.path.join(os.getcwd(), 'locale', language, 'LC_MESSAGES', app_settings.PO_FILES_NAME)
            po_files_paths.append((po_file, language))
    
    if not po_files_paths:
        return False, "No po files detected in the project."
            
   
    for po_file_details in po_files_paths:
       
        po_file_path = po_file_details[0]
        po_file_language = po_file_details[1]
        
        spinner = Halo(text=f"Processing {po_file_language} PO file", spinner='dots')
        spinner.start()
     
        lines = None
        with open(po_file_path, 'r', encoding='utf-8') as po_file:
            lines = po_file.readlines()
                
        if lines:
            with open(po_file_path, 'w', encoding='utf-8') as po_file:
                result, processed_list = lines_processor(lines=lines, lan=po_file_language, 
                                               update_already_translated=translate_existed, 
                                               resolve_fuzzy=resolve_fuzzy)
                for line in processed_list:
                    po_file.write(line) 
                    
            if result:
                command.stdout.write(command.style.SUCCESS(f"\n{po_file_language} Po file have been processed ... Ok"))
            else:
                command.stdout.write(command.style.ERROR(f"\nUnexpected issue while processing {po_file_language} Po file, all changes have been reverted."))
        else:
            command.stdout.write(command.style.WARNING(f"\n{po_file_language} Po file is empty .. no translations were added."))

        spinner.stop()
    
    command.stdout.write("\n\nProcess completed .. you should compile your messages now to reflect new translations.\n\n")


from setuptools import setup, find_packages


long_description = ''
with open("README.md", "r") as fh:
    long_description = fh.read()

setup(
    name='django_po_translator',
    version='0.1.2',
    description='Translate Django PO files and resolve fuzziness with ease.',
    long_description=long_description,
    long_description_content_type="text/markdown",
    packages=find_packages(),
    install_requires=[
        'Django>=2.2.0',
        'beautifulsoup4',
        'halo',
    ],
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
    ],
    url='https://github.com/waseem-hamoolaila/django-po-translator',
    author='Waseem Hamoolaila',
    author_email='waseem.97.laila@gmail.com',
)